/* vi:set cindent tabstop=2: */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>

#include "mhihttp.h"

char *usage =
  "Usage: %s [-dhv] [-r documentroot] [-p port] [-i indexfile] [-m mime.types]\n"
  "            [-t defaultmime]"
  "\n" "Please report bugs to mhi@penguin.cz\n";

char docroot[0x100];
char defaultmime[0x100];
int port;
int daemonize;
int verbose;
struct indexfile *indexfiles;
struct mimetype *mimetypes;

int
main (int argc, char **argv)
{
  int ch;
  int server_s;
  int s;
  struct sockaddr_in server_sockaddr;
  struct sockaddr_in remote_addr;
  int remote_addrlen = sizeof (remote_addr);
  int sock_opt = 1;

  port = 80;
  docroot[0] = 0;
  defaultmime[0] = 0;
  daemonize = 0;
  verbose = 0;
  indexfiles = 0;
  mimetypes = 0;

  signal (SIGCHLD, removezombies);

  while ((ch = getopt (argc, argv, "hr:p:dvi:m:")) != -1)
    {
      switch (ch)
	{
	case 'h':
	  fprintf (stderr, usage, argv[0]);
	  exit (0);
	case 'r':
	  strcpy (docroot, optarg);	/* XXX */
	  break;
	case 'p':
	  port = atoi (optarg);
	  break;
	case 'd':
	  daemonize = 1;
	  break;
	case 'v':
	  verbose = 1;
	  break;
	case 'i':
	  add_indexfile (optarg);
	  break;
	case 'm':
	  read_mimetypes (optarg);
	  break;
	case 't':
	  strcpy (defaultmime, optarg);
	  break;
	default:
	  fprintf (stderr, usage, argv[0]);
	  exit (1);
	  break;
	}
    }

  if (argc - optind != 0)
    {
      fprintf (stderr, usage, argv[0]);
      exit (1);
    }

  if (docroot[0] == 0)
    {
      fprintf (stderr, "%s: Document root not specified\n", argv[0]);
      exit (1);
    }

  if (indexfiles == NULL)
    add_indexfile ("index.html");

  if (mimetypes == NULL)
    read_mimetypes ("/etc/mime.types");

  if (mimetypes == NULL)
    {
      addmime ("text/html", "html");
      addmime ("text/html", "htm");
      addmime ("text/plain", "txt");
      addmime ("image/gif", "gif");
      addmime ("image/jpeg", "jpg");
      addmime ("image/jpeg", "jpeg");
    }

  if (defaultmime[0] == 0)
    strcpy (defaultmime, "text/plain");

  if ((server_s = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1)
    {
      perror ("socket");
      exit (1);
    }

  if ((setsockopt (server_s, SOL_SOCKET, SO_REUSEADDR, (void *) &sock_opt,
		   sizeof (sock_opt))) == -1)
    {
      perror ("setsockopt");
      exit (1);
    }

  server_sockaddr.sin_family = AF_INET;
  server_sockaddr.sin_addr.s_addr = htonl (INADDR_ANY);
  server_sockaddr.sin_port = htons (port);

  if (bind (server_s, (struct sockaddr *) &server_sockaddr,
	    sizeof (server_sockaddr)) == -1)
    {
      perror ("bind");
      exit (1);
    }

  if (listen (server_s, 10) == -1)
    {
      perror ("listen");
      exit (1);
    }

  if (daemonize)
    if (fork ())
      exit (0);

  while (1)
    {
      s =
	accept (server_s, (struct sockaddr *) &remote_addr, &remote_addrlen);
      if (s < 0)
	{
	  perror ("accept");
	  exit (1);
	}
      if (fork () == 0)
	{
	  do_http (s, (struct sockaddr *) &remote_addr);
	}
      else
	{
	  close (s);
	}
    }

  return 0;
}

void
add_indexfile (char *name)
{
  struct indexfile *i;

  i = (struct indexfile *) malloc (sizeof (struct indexfile));
  if (!i)
    {
      fprintf (stderr, "add_indexfile: Not enough memory\n");
      exit (1);
    }
  i->name = strdup (name);
  if (!i->name)
    {
      fprintf (stderr, "add_indexfile: Not enough memory\n");
      exit (1);
    }
  i->next = indexfiles;
  indexfiles = i;
}
